﻿using AutoMapper;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Extensions.DependencyInjection;
using System.Web.Mvc;
using WebApplication5.Models;
using WebApplication5.Templates;
using System.IO;
using System.Configuration;

namespace WebApplication5.Controllers
{
    public class CRUDController : Controller
    {
        private readonly IMapper _mapper;
        private TestsDbContext _db;

        public CRUDController(IServiceCollection serviceDescriptors)
        {
        }

        public CRUDController(TestsDbContext db, IMapper mapper)
        {
            _mapper = mapper;
            _db = db;
        }
        public ActionResult Index()
        {
            return View();
        }

        #region Views with lists
        public ActionResult Categories()
        {
            var result = _mapper.ProjectTo<CategoryViewModel>(_db.Categories);
            return View(result.ToList());
        }
        public ActionResult Questions(int categoryId)
        {
            var result = _mapper.ProjectTo<QuestionViewModel>(_db.Questions.Where(q => q.CategoryId == categoryId));
            var response = new QuestionView()
            {
                Questions = result.ToList(),
                CategoryId = categoryId
            };
            return View(response);
        }
        public ActionResult Answers(int questionId)
        {
            var result = _mapper.ProjectTo<AnswerViewModel>(_db.Answers.Where(q => q.QuestionId == questionId));
            var catId = _db.Questions.First(q => q.Id == questionId).CategoryId;
            var response = new AnswerView()
            {
                Answers = result.ToList(),
                QuestionId = questionId,
                CategoryId = catId
            };
            return View(response);
        }
        #endregion

        #region Posts
        public ActionResult PostCategory(CategoryViewModel category)
        {           var cat = _mapper.Map<Category>(category);
                        _db.Categories.Add(cat);
                        _db.SaveChanges();
         
                _db.SaveChanges();

            return RedirectToAction("Categories");
        }
        public ActionResult PostQuestion(QuestionViewModel question, int categoryId)
        {
            var q = _mapper.Map<Question>(question);
            q.CategoryId = categoryId;
            _db.Questions.Add(q);
            _db.SaveChanges();

            return RedirectToAction("Questions", new { categoryId });
        }
        public ActionResult PostAnswer(AnswerViewModel answer, int questionId)
        {
            var a = _mapper.Map<Answer>(answer);
            a.QuestionId = questionId;
            _db.Answers.Add(a);
            _db.SaveChanges();

            return RedirectToAction("Answers", new { questionId });
        }
        #endregion

        #region Deletes
        public ActionResult DeleteAnswer(int answerId)
        {
            var a = _db.Answers.FirstOrDefault(qu => qu.Id == answerId);
            var questionId = a.QuestionId;
            _db.Answers.Remove(a);
            _db.SaveChanges();

            return RedirectToAction("Answers", new { questionId });
        }
        public ActionResult DeleteCategory(int categoryId)
        {
            var c = _db.Categories.Include("Questions").FirstOrDefault(cat => cat.Id == categoryId);

            if (c == null)
                return null;

            foreach (var q in c.Questions)
            {
                q.CategoryId = 1;//Unknown, pre initialized in the DB.
            }

            _db.Categories.Remove(c);
            _db.SaveChanges();

            return RedirectToAction("Categories");
        }
        public ActionResult DeleteQuestion(int questionId)
        {
            var q = _db.Questions.Include("Answers").FirstOrDefault(qu => qu.Id == questionId);
            var categoryId = q.CategoryId;
            if (q == null)
                return null;

            foreach (var a in q.Answers)
                _db.Answers.Remove(a);

            _db.Questions.Remove(q);
            _db.SaveChanges();

            return RedirectToAction("Questions", new { categoryId });
        }
        #endregion

        #region CreateTemplate
        public ActionResult AllForTemplate()
        {
            var result = _mapper.ProjectTo<CategoryViewModel>(_db.Categories);
            return View(result.ToList());

        }
        public ActionResult GenerateTemplate(List<CategoryViewModel> categoryViewModels,int numerOfExamples, string testName)
        {
            var templateViewModel = categoryViewModels.Select(obj => new TemplateViewModel()
            {
                CategoryId = obj.Id.Value,
                QuestionsIds = obj.Questions.Where(q=>q.Checked).Select(q=>q.Id.Value).ToList()
            }).ToList();

            var templatesToGenerate = new TemplatesOrder()
            {
                Categories = templateViewModel,
                NumberOfExamples = numerOfExamples,
                TestName = testName
            };
            var generator = new TemplateGenerator(_db, templatesToGenerate);
            generator.ProcessOrder();

            return RedirectToAction("AllForTemplate");
        }
        #endregion

        #region Check Templates
        public ActionResult Tests()
        {
            var path = string.Format("{0}{1}", ConfigurationManager.AppSettings["TestsFolder"], "toCheck");
            var listOfFoldersWithTests =  Directory.GetDirectories(path);
            return View(listOfFoldersWithTests.ToList());
        }

        public ActionResult CheckTest(string path)
        {
            var x = new CheckCards(_db);
            var results = x.CheckFolder(path);

            return View(results);
        }
        #endregion
    }
}
