﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using WebApplication5.Models.Business;
using Emgu.CV;
using Emgu.Util;
using Emgu.CV.Structure;
using System.IO;
using ImageMagick;
using System.Drawing;
using WebApplication5.Models;

namespace WebApplication5.Templates
{
    public class CheckCards
    {
        private TestsDbContext _db;
        public CheckCards(TestsDbContext db)
        {
            _db = db;
        }
        public List<Result> CheckFolder(string folder)
        {
            var results = new List<Result>();
            string[] fileEntries = Directory.GetFiles(folder);

            foreach (string fileName in fileEntries.Where(f => f.EndsWith(".pdf")))
            {
                var answers = ProcessFile(fileName);
                var questionsIds = answers.Select(a => a.QuestionId).ToList();
                var questions = _db.Questions.Include("Answers").Where(q => questionsIds.Contains(q.Id)).ToList();

                var points = 0;
                var maxPoints = questions.Sum(q => q.DefaultScore);

                foreach (var q in questions)
                {
                    var ta = answers.First(a => a.QuestionId == q.Id).Answers.Where(a => a.Checked).Select(a => a.AnswerNumber).ToList();
                    var correctRelativeAnswers = new List<int>();
                    for (int i = 0; i < q.Answers.Count; i++)
                    {
                        if (q.Answers.ElementAt(i).IsCorrect)
                            correctRelativeAnswers.Add(i);
                    }
                    if (ta.SequenceEqual(correctRelativeAnswers))
                        points += q.DefaultScore;
                }
                results.Add(new Result()
                {
                    AnswerFileName = fileName,
                    Points = points,
                    PointsMax = maxPoints
                });
            }
            return results;
        }
        private List<TemplateAnswer> ProcessFile(string fileName)
        {
            fileName = PDFtoIMG(fileName);
            var tr = ImgToGray(fileName);
            var qrDetector = new QRCodeDetector();
            Mat x = new Mat();
            Mat y = new Mat();
            qrDetector.Detect(tr, x);
          
            var qrCode = qrDetector.Decode(tr, x, y);
            var greyThresh = RotateImage(tr,x);
            greyThresh.ThresholdBinary(new Gray(1), new Gray(255));

            var questionsIds = qrCode.Split('_').Select(i => int.Parse(i));
            var numbOfQuestions = questionsIds.Count();
            var lcolumn = numbOfQuestions - (numbOfQuestions / 2);
            var lcolumnq = questionsIds.Take(lcolumn).ToList();
            var rcolumnq = questionsIds.Skip(lcolumn).ToList();


            var widthl = _db.Questions.Where(q => lcolumnq.Contains(q.Id)).Max(q => q.Answers.Count);
            var widthr = _db.Questions.Where(q => rcolumnq.Contains(q.Id)).Max(q => q.Answers.Count);

            var ans1 = GetAnswers(greyThresh, lcolumnq, false, 480, widthl);
            var ans2 = GetAnswers(greyThresh, rcolumnq, true, 480, widthr);

            return ans1.Concat(ans2).ToList();
        }




        private int GetHeight(LineSegment2D[] lines, int questionNumbers)
        {
            return Math.Abs(GetTopLine(lines) - GetBottomLine(lines)) / questionNumbers;
        }
        private double LineAvg(LineSegment2D[] lines)
        {
            return lines.Average(l => Math.Min(l.P2.Y, l.P1.Y));
        }

        private int GetTopLine(LineSegment2D[] lines)
        {
            return lines.Min(l => Math.Min(l.P2.Y, l.P1.Y));
        }
        private int GetBottomLine(LineSegment2D[] lines)
        {
            return lines.Max(l => Math.Max(l.P2.Y, l.P1.Y));
        }
        private int GetLeftLine(LineSegment2D[] lines)
        {
            return lines.Min(l => Math.Min(l.P2.X, l.P1.X));
        }
        private int GetRightLine(LineSegment2D[] lines)
        {
            return lines.Max(l => Math.Max(l.P2.X, l.P1.X));
        }

        private List<TemplateAnswer> GetAnswers(Image<Gray, byte> image, List<int> questionsIds, bool rightcolumn, int idetiferHeight, int ansNumber)
        {
            var result = new List<TemplateAnswer>();
            image.ROI = new Rectangle();

            if (rightcolumn)
                image.ROI = new Rectangle(image.Size.Width / 2, idetiferHeight, image.Size.Width / 2, image.Size.Height - idetiferHeight);
            else
                image.ROI = new Rectangle(0, idetiferHeight, image.Size.Width / 2, image.Size.Height- idetiferHeight);

            //image.Save(@"C:\Users\Janek\Desktop\Tests\scans\half_" + rightcolumn + ".jpg");
            var lines = GetHorizontal(image);
            var left = GetLeftLine(lines);
            var right = GetRightLine(lines);
            var top = GetTopLine(lines);
            var height = GetHeight(lines, questionsIds.Count);
            var widthPerAnswer = (right - left) / ansNumber;

            image.ROI = new Rectangle();
            var rec = new Rectangle(rightcolumn ? image.Size.Width / 2 + left : left, idetiferHeight + top, right - left, height * questionsIds.Count);
            image.Draw(rec, new Gray(255), 10);
            //image.Save(@"C:\Users\Janek\Desktop\Tests\scans\" + rightcolumn + "lines.jpg");

            for (int i = 0; i < questionsIds.Count; i++)
            {
                var xStart = left;
                if (rightcolumn)
                {
                    image.ROI = new Rectangle();
                    xStart = xStart + image.Size.Width / 2;
                }
                var qA = new TemplateAnswer()
                {
                    QuestionId = questionsIds.ElementAt(i),
                    Answers = new List<SimpleAnswer>()
                };
                for (int j = 0; j < ansNumber; j++)
                {
                    var a = new SimpleAnswer()
                    {
                        AnswerNumber = j,
                        Checked = false
                    };
                    var recc = new Rectangle(xStart, idetiferHeight + top + height * i, widthPerAnswer, height);
                    xStart = xStart + widthPerAnswer;

                    image.ROI = recc;    
                    var pixels = (float)image.ROI.Size.Height * image.ROI.Width;
                    var nonZero = (float)image.CountNonzero()[0];

                    //image.Save(@"C:\Users\Janek\Desktop\Tests\scans\" + i + j + rightcolumn + "boxes.jpg");
                    var ratio = (pixels - nonZero) / pixels;
                    if (ratio > 0.1)
                    {
                        a.Checked = true;
                        //image.Save(@"C:\Users\Janek\Desktop\Tests\scans\" + rightcolumn + ratio + "lines.jpg");
                    }

                    qA.Answers.Add(a);

                }
                result.Add(qA);
            }
            return result;
        }
        public LineSegment2D[] GetHorizontal(Image<Gray, byte> image)
        {
            var lines =  image.HoughLines(200, 100, 1, Math.PI / 90, 50, 50, 5)[0]
                .Where(l => l.Direction.X != 0 && l.Direction.Y == 0).ToArray();

            var avg = LineAvg(lines);
            var topLine = GetTopLine(lines);
            var avgDist = avg - topLine;

            lines = lines.Where(l => Math.Max(l.P1.Y, l.P2.Y) < (2 * avgDist) + topLine + 100)
                .ToArray();

            return lines;
        }

        private Image<Gray, byte> ImgToGray(string fileName)
        {
            Image<Bgr, byte> answersCard = new Image<Bgr, byte>(fileName);
            Image<Gray, byte> grayImage = answersCard.Convert<Gray, byte>();
           
            return grayImage;
        }
        private Image<Gray, byte> RotateImage(Image<Gray, byte> image, Mat x)
        {
            var z = x.GetData();
            var pointOfQr = new Point()
            {
                X = int.Parse(z.GetValue(0, 0, 0).ToString()),
                Y = int.Parse(z.GetValue(0, 0, 1).ToString())
            };
            var imgPoints = new List<Point>()
            {
                new Point(0, 0), //left top
                new Point(image.Width, 0), //right top
                new Point(image.Width, image.Height), //right bottom
                new Point(0, image.Height) //left bottom
            };
            var distances = new List<double>()
            {
                PointsDistance(imgPoints[0], pointOfQr),
                PointsDistance(imgPoints[1], pointOfQr),
                PointsDistance(imgPoints[2], pointOfQr),
                PointsDistance(imgPoints[3], pointOfQr)
            };

            var i = distances.IndexOf(distances.Min());

            if (i == 0)
                image.Rotate(90, new Gray(255), false);
            if (i == 1)
            { }//do nothing it's in top right cornet
            if (i == 2)
                image.Rotate(270, new Gray(255), false);
            if (i == 3)
            {
                image = image.Rotate(180, new Gray(255), false);
                //image.Save(@"C:\Users\Janek\Desktop\Tests\scans\" + "rotated.jpg");
            }
            return image.ThresholdToZero(new Gray(150));
        }


        private double PointsDistance(Point x, Point y)
        {
            return Math.Abs(x.X - y.X) ^ 2 + Math.Abs(x.Y - y.Y) ^ 2;
        }

        public void CheckRotation(Image<Gray, byte> image, Mat square)
        {
            double x = 0, y = 0;
            var arr = square.GetData();
            for (int i = 0; i < 4; i++)
            {
                x += x + (double)arr.GetValue(2 * i);
            }
            for (int i = 0; i < 4; i++)
            {
                y += y + (double)arr.GetValue(2 * i + 1);
            }
            x = x / 4;
            y = y / 4;

            var xMax = image.Size.Width;
            var yMax = image.Size.Height;

            var distanceFromCorners = new List<int>();
            var left = Math.Abs(xMax - x);

            image.Rotate(180, new Gray(0));
        }
        private string PDFtoIMG(string fileName)
        {
            var settings = new MagickReadSettings();
            // Settings the density to 300 dpi will create an image with a better quality
            settings.Density = new Density(300);

            using (var images = new MagickImageCollection())
            {
                // Add all the pages of the pdf file to the collection
                images.Read(fileName, settings);
                fileName = fileName.Replace(".pdf", ".jpg");
                using (var horizontal = images.AppendHorizontally())
                {
                    horizontal.Write(fileName);
                }
            }
            return fileName;
        }
    }
}